#!/usr/bin/env python

##############################################################################
#	This software was developed by the University of Tennessee as part of the
#	Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
#	project funded by the US National Science Foundation.
#
#	If you use DANSE applications to do scientific research that leads to
#	publication, we ask that you acknowledge the use of the software with the
#	following sentence:
#
#	"This work benefited from DANSE software developed under NSF award DMR-0520547."
#
#	copyright 2008, University of Tennessee
##############################################################################


""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY include/binaryHS.h
         AND RE-RUN THE GENERATOR SCRIPT

"""

from sans.models.BaseComponent import BaseComponent
from sans.models.sans_extension.c_models import CBinaryHSModel
import copy    

def create_BinaryHSModel():
    obj = BinaryHSModel()
    #CBinaryHSModel.__init__(obj) is called by BinaryHSModel constructor
    return obj

class BinaryHSModel(CBinaryHSModel, BaseComponent):
    """ 
    Class that evaluates a BinaryHSModel model. 
    This file was auto-generated from include/binaryHS.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         l_radius        = 100.0 [A]
         s_radius        = 25.0 [A]
         vol_frac_ls     = 0.1 
         vol_frac_ss     = 0.2 
         ls_sld          = 3.5e-06 [1/A^(2)]
         ss_sld          = 5e-07 [1/A^(2)]
         solvent_sld     = 6.36e-06 [1/A^(2)]
         background      = 0.001 [1/cm]

    """
        
    def __init__(self):
        """ Initialization """
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        #apply(CBinaryHSModel.__init__, (self,)) 
        CBinaryHSModel.__init__(self)
        
        ## Name of the model
        self.name = "BinaryHSModel"
        ## Model description
        self.description =""" Model parameters: l_radius : large radius of binary hard sphere
		s_radius : small radius of binary hard sphere
		vol_frac_ls : volume fraction of large spheres
		vol_frac_ss : volume fraction of small spheres
		ls_sld: large sphere  scattering length density
		ss_sld: small sphere scattering length density
		solvent_sld: solvent scattering length density
		background: incoherent background"""
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['l_radius'] = ['[A]', None, None]
        self.details['s_radius'] = ['[A]', None, None]
        self.details['vol_frac_ls'] = ['', None, None]
        self.details['vol_frac_ss'] = ['', None, None]
        self.details['ls_sld'] = ['[1/A^(2)]', None, None]
        self.details['ss_sld'] = ['[1/A^(2)]', None, None]
        self.details['solvent_sld'] = ['[1/A^(2)]', None, None]
        self.details['background'] = ['[1/cm]', None, None]

        ## fittable parameters
        self.fixed=['l_radius.width', 's_radius.width']
        
        ## non-fittable parameters
        self.non_fittable = []
        
        ## parameters with orientation
        self.orientation_params = []

    def __setstate__(self, state):
        """
        restore the state of a model from pickle
        """
        self.__dict__, self.params, self.dispersion = state
        
    def __reduce_ex__(self, proto):
        """
        Overwrite the __reduce_ex__ of PyTypeObject *type call in the init of 
        c model.
        """
        state = (self.__dict__, self.params, self.dispersion)
        return (create_BinaryHSModel,tuple(), state, None, None)
        
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(BinaryHSModel())   
       	
   
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        
        return CBinaryHSModel.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        
        return CBinaryHSModel.runXY(self, x)
        
    def evalDistribution(self, x=[]):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CBinaryHSModel.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CBinaryHSModel.calculate_ER(self)
        
    def calculate_VR(self):
        """ 
        Calculate the volf ratio for P(q)*S(q)
        
        :return: the value of the volf ratio
        
        """       
        return CBinaryHSModel.calculate_VR(self)
              
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CBinaryHSModel.set_dispersion(self, parameter, dispersion.cdisp)
        
   
# End of file
