#!/usr/bin/env python

##############################################################################
#	This software was developed by the University of Tennessee as part of the
#	Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
#	project funded by the US National Science Foundation.
#
#	If you use DANSE applications to do scientific research that leads to
#	publication, we ask that you acknowledge the use of the software with the
#	following sentence:
#
#	"This work benefited from DANSE software developed under NSF award DMR-0520547."
#
#	copyright 2008, University of Tennessee
##############################################################################


""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY include/lamellarPC.h
         AND RE-RUN THE GENERATOR SCRIPT

"""

from sans.models.BaseComponent import BaseComponent
from sans.models.sans_extension.c_models import CLamellarPCrystalModel
import copy    

def create_LamellarPCrystalModel():
    obj = LamellarPCrystalModel()
    #CLamellarPCrystalModel.__init__(obj) is called by LamellarPCrystalModel constructor
    return obj

class LamellarPCrystalModel(CLamellarPCrystalModel, BaseComponent):
    """ 
    Class that evaluates a LamellarPCrystalModel model. 
    This file was auto-generated from include/lamellarPC.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         scale           = 1.0 
         thickness       = 33.0 [A]
         Nlayers         = 20.0 
         spacing         = 250.0 [A]
         pd_spacing      = 0.0 
         sld_layer       = 1e-06 [1/A^(2)]
         sld_solvent     = 6.34e-06 [1/A^(2)]
         background      = 0.0 [1/cm]

    """
        
    def __init__(self):
        """ Initialization """
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        #apply(CLamellarPCrystalModel.__init__, (self,)) 
        CLamellarPCrystalModel.__init__(self)
        
        ## Name of the model
        self.name = "LamellarPCrystalModel"
        ## Model description
        self.description ="""[Lamellar ParaCrystal Model] Parameter Definitions: scale = scale factor,
		background = incoherent background
		thickness = lamellar thickness,
		sld_layer = layer scattering length density ,
		sld_solvent = solvent scattering length density.
		Nlayers = no. of lamellar layers
		spacing = spacing between layers
		pd_spacing = polydispersity of spacing
		Note: This model can be used for large
		multilamellar vesicles.
		"""
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['scale'] = ['', None, None]
        self.details['thickness'] = ['[A]', None, None]
        self.details['Nlayers'] = ['', None, None]
        self.details['spacing'] = ['[A]', None, None]
        self.details['pd_spacing'] = ['', None, None]
        self.details['sld_layer'] = ['[1/A^(2)]', None, None]
        self.details['sld_solvent'] = ['[1/A^(2)]', None, None]
        self.details['background'] = ['[1/cm]', None, None]

        ## fittable parameters
        self.fixed=['thickness.width']
        
        ## non-fittable parameters
        self.non_fittable = []
        
        ## parameters with orientation
        self.orientation_params = []

    def __setstate__(self, state):
        """
        restore the state of a model from pickle
        """
        self.__dict__, self.params, self.dispersion = state
        
    def __reduce_ex__(self, proto):
        """
        Overwrite the __reduce_ex__ of PyTypeObject *type call in the init of 
        c model.
        """
        state = (self.__dict__, self.params, self.dispersion)
        return (create_LamellarPCrystalModel,tuple(), state, None, None)
        
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(LamellarPCrystalModel())   
       	
   
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        
        return CLamellarPCrystalModel.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        
        return CLamellarPCrystalModel.runXY(self, x)
        
    def evalDistribution(self, x=[]):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CLamellarPCrystalModel.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CLamellarPCrystalModel.calculate_ER(self)
        
    def calculate_VR(self):
        """ 
        Calculate the volf ratio for P(q)*S(q)
        
        :return: the value of the volf ratio
        
        """       
        return CLamellarPCrystalModel.calculate_VR(self)
              
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CLamellarPCrystalModel.set_dispersion(self, parameter, dispersion.cdisp)
        
   
# End of file
