#!/usr/bin/env python

##############################################################################
#	This software was developed by the University of Tennessee as part of the
#	Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
#	project funded by the US National Science Foundation.
#
#	If you use DANSE applications to do scientific research that leads to
#	publication, we ask that you acknowledge the use of the software with the
#	following sentence:
#
#	"This work benefited from DANSE software developed under NSF award DMR-0520547."
#
#	copyright 2008, University of Tennessee
##############################################################################


""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY include/massfractal.h
         AND RE-RUN THE GENERATOR SCRIPT

"""

from sans.models.BaseComponent import BaseComponent
from sans.models.sans_extension.c_models import CMassFractalModel
import copy    

def create_MassFractalModel():
    obj = MassFractalModel()
    #CMassFractalModel.__init__(obj) is called by MassFractalModel constructor
    return obj

class MassFractalModel(CMassFractalModel, BaseComponent):
    """ 
    Class that evaluates a MassFractalModel model. 
    This file was auto-generated from include/massfractal.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         scale           = 1.0 
         radius          = 10.0 [A]
         mass_dim        = 1.9 
         co_length       = 100.0 [A]
         background      = 0.0 

    """
        
    def __init__(self):
        """ Initialization """
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        #apply(CMassFractalModel.__init__, (self,)) 
        CMassFractalModel.__init__(self)
        
        ## Name of the model
        self.name = "MassFractalModel"
        ## Model description
        self.description =""" The scattering intensity  I(x) = scale*P(x)*S(x) + background, where
		scale = scale_factor  * V * delta^(2)
		p(x)=  F(x*radius)^(2)
		F(x) = 3*[sin(x)-x cos(x)]/x**3
		S(x) = [(gamma(Dm-1)*colength^(Dm-1)*[1+(x^2*colength^2)]^((1-Dm)/2)
		* sin[(Dm-1)*arctan(x*colength)])/x]
		where delta = sldParticle -sldSolv.
		radius       =  Particle radius
		mass_dim  =  Mass fractal dimension
		co_length  =  Cut-off length
		background   =  background
		Ref.:Mildner, Hall,J Phys D Appl Phys(1986), 9, 1535-1545
		Note I: This model is valid for 1<mass_dim<6.
		Note II: This model is not in absolute scale."""
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['scale'] = ['', None, None]
        self.details['radius'] = ['[A]', None, None]
        self.details['mass_dim'] = ['', None, None]
        self.details['co_length'] = ['[A]', None, None]
        self.details['background'] = ['', None, None]

        ## fittable parameters
        self.fixed=[]
        
        ## non-fittable parameters
        self.non_fittable = []
        
        ## parameters with orientation
        self.orientation_params = []

    def __setstate__(self, state):
        """
        restore the state of a model from pickle
        """
        self.__dict__, self.params, self.dispersion = state
        
    def __reduce_ex__(self, proto):
        """
        Overwrite the __reduce_ex__ of PyTypeObject *type call in the init of 
        c model.
        """
        state = (self.__dict__, self.params, self.dispersion)
        return (create_MassFractalModel,tuple(), state, None, None)
        
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(MassFractalModel())   
       	
   
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        
        return CMassFractalModel.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        
        return CMassFractalModel.runXY(self, x)
        
    def evalDistribution(self, x=[]):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CMassFractalModel.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CMassFractalModel.calculate_ER(self)
        
    def calculate_VR(self):
        """ 
        Calculate the volf ratio for P(q)*S(q)
        
        :return: the value of the volf ratio
        
        """       
        return CMassFractalModel.calculate_VR(self)
              
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CMassFractalModel.set_dispersion(self, parameter, dispersion.cdisp)
        
   
# End of file
