#!/usr/bin/env python

##############################################################################
#	This software was developed by the University of Tennessee as part of the
#	Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
#	project funded by the US National Science Foundation.
#
#	If you use DANSE applications to do scientific research that leads to
#	publication, we ask that you acknowledge the use of the software with the
#	following sentence:
#
#	"This work benefited from DANSE software developed under NSF award DMR-0520547."
#
#	copyright 2008, University of Tennessee
##############################################################################


""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY include/parallelepiped.h
         AND RE-RUN THE GENERATOR SCRIPT

"""

from sans.models.BaseComponent import BaseComponent
from sans.models.sans_extension.c_models import CParallelepipedModel
import copy    

def create_ParallelepipedModel():
    obj = ParallelepipedModel()
    #CParallelepipedModel.__init__(obj) is called by ParallelepipedModel constructor
    return obj

class ParallelepipedModel(CParallelepipedModel, BaseComponent):
    """ 
    Class that evaluates a ParallelepipedModel model. 
    This file was auto-generated from include/parallelepiped.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         scale           = 1.0 
         short_a         = 35.0 [A]
         short_b         = 75.0 [A]
         long_c          = 400.0 [A]
         sldPipe         = 6.3e-06 [1/A^(2)]
         sldSolv         = 1e-06 [1/A^(2)]
         background      = 0.0 [1/cm]
         parallel_theta  = 0.0 [deg]
         parallel_phi    = 0.0 [deg]
         parallel_psi    = 0.0 [deg]

    """
        
    def __init__(self):
        """ Initialization """
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        #apply(CParallelepipedModel.__init__, (self,)) 
        CParallelepipedModel.__init__(self)
        
        ## Name of the model
        self.name = "ParallelepipedModel"
        ## Model description
        self.description =""" Form factor for a rectangular solid with uniform scattering length density.
		
		scale:Scale factor
		short_a: length of short edge  [A]
		short_b: length of another short edge [A]
		long_c: length of long edge  of the parallelepiped [A]
		sldPipe: Pipe_sld
		sldSolv: solvent_sld
		background:Incoherent Background [1/cm]"""
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['scale'] = ['', None, None]
        self.details['short_a'] = ['[A]', None, None]
        self.details['short_b'] = ['[A]', None, None]
        self.details['long_c'] = ['[A]', None, None]
        self.details['sldPipe'] = ['[1/A^(2)]', None, None]
        self.details['sldSolv'] = ['[1/A^(2)]', None, None]
        self.details['background'] = ['[1/cm]', None, None]
        self.details['parallel_theta'] = ['[deg]', None, None]
        self.details['parallel_phi'] = ['[deg]', None, None]
        self.details['parallel_psi'] = ['[deg]', None, None]

        ## fittable parameters
        self.fixed=['short_a.width', 'short_b.width', 'long_c.width', 'parallel_phi.width', 'parallel_psi.width', 'parallel_theta.width']
        
        ## non-fittable parameters
        self.non_fittable = []
        
        ## parameters with orientation
        self.orientation_params = ['parallel_phi', 'parallel_psi', 'parallel_theta', 'parallel_phi.width', 'parallel_psi.width', 'parallel_theta.width']

    def __setstate__(self, state):
        """
        restore the state of a model from pickle
        """
        self.__dict__, self.params, self.dispersion = state
        
    def __reduce_ex__(self, proto):
        """
        Overwrite the __reduce_ex__ of PyTypeObject *type call in the init of 
        c model.
        """
        state = (self.__dict__, self.params, self.dispersion)
        return (create_ParallelepipedModel,tuple(), state, None, None)
        
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(ParallelepipedModel())   
       	
   
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        
        return CParallelepipedModel.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        
        return CParallelepipedModel.runXY(self, x)
        
    def evalDistribution(self, x=[]):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CParallelepipedModel.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CParallelepipedModel.calculate_ER(self)
        
    def calculate_VR(self):
        """ 
        Calculate the volf ratio for P(q)*S(q)
        
        :return: the value of the volf ratio
        
        """       
        return CParallelepipedModel.calculate_VR(self)
              
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CParallelepipedModel.set_dispersion(self, parameter, dispersion.cdisp)
        
   
# End of file
